#include "h/types.h"
#include <string.h>
#include "h/emuFileHandling.h"
#include "../nes/h/nesGeneral.h"
#include "../emu/h/emuGeneral.h"

namespace Reminesce
{

u8 *prgromPtr, *chrromPtr;
ui imageSize;
u16 *loggedImage = 0; u8 rwMode, logMode = 0;

char szFilename[260];
struct iNesHeader header;

int loadPath()
{
	//will load in header & display warning msg
	u8 head[16];
	ui size;	

	if (imageLoaded)
	{
		deletePrgChrMemory();
	}

	imageLoaded = 0;

	size = LoadHeader(head);
	if (!size)
		return 1;										//Couldnt open
	
	if (checkHeadInfo(checkHeader(head, &size)))
		return 2;
	
	loadImage(size);
	if (postLoad())
		return 3;

	imageLoaded = 1;
	HardResetNes();

	return 0;
}

u8 checkHeader(u8 *head, ui *size)
{
	ui size2, temp;
	s32 info;

	if (!( (head[0] == 'N') && (head[1]== 'E') && ((head[2] == 'S') | (head[2]== 'Z')) && (head[3] == 0x1A) ))
		return 2;

	header.prg		= head[4];
	header.chr		= head[5];
	header.vmirror	= head[6] & 1;
	header.sram		= head[6] & 2;
	header.trainer	= head[6] & 4;
	header.fourScreenMirroring	= head[6] & 8;
	header.mapNumber= (head[6]>>4) | (head[7] & 0xF0);
	if (header.footerText)
	{
		delete [] header.footerText;
		header.footerText = 0;
	}
	header.footer = 0;									//will check for footer below

	if (!header.prg)									//no prgrom ?!
		return 3;

	temp = 1;
	size2 = header.prg;
	while (size2 != 1)
	{
		size2 >>= 1;
		temp <<= 1;
	}
	header.prg8 = temp*16-1;
	if (header.prg != temp)								//prg is not multiple of 2
		return 4;
	
	if (header.chr > 1)										//find amount to & bank number
	{
		temp = 1;
		size2 = header.chr-1;
		while (size2 != 1)
		{
			size2 >>= 1;
			temp <<= 1;
		};
		header.chr1 = (temp*2)*8 -1;
	}
	else
	{
		header.chr1 = (header.chr*8)-1;
	}

	size2 = *size;
	*size = (header.prg*1024*16) + (header.chr*1024*8);

	info  = size2 - *size - 16;
	header.junk = info;
	info -= (header.trainer ? 512 : 0);

	if (info > 0)										//Unknown, Trainer or Footer in file
	{
		if ((info == 128) || (info == 127))
			info = 5;									//Footer
		else
			info = 6;									//garabage at end?
		header.footer = 1;
	}
	else if (info < 0)									//missing somthing
	{
		if (info == -512)
			if (!header.trainer)
				info = 7;			//no trainer but header says thinks there is, ask to fix header?
			else
				info = 8;								//huh!
		else
			info = 9;									//Missing somthing.
	}
	else												//File size exactly as expected
		info = 0;

	if (head[15] == '!')
		info |= 0x20;									// ! of "DiskDude!"

	for (temp = 8; temp<16; temp++)						//check for other junk
	{
		if (head[temp] != 0)
		{
			info |= 0x40;
			break;
		}
	}

	return (u8)info;
	/*
	2 Bad header sig
	3 no prg rom ?
	4 prg isnt multiple of 2
	5 average footer found
	6 garbage at end
	7 no trainer but it thinks there is
	8 missing stuff
	9 missing stuff
	D5 DiskDude! <--crap
	D6 junk in header
	*/
}

void setRegionMode(char *szFilename)
{
	/*			Sets up NTSC/PAL based on whats inside of () in the filename		*/
	char szIndx;
	short szLen = strlen(szFilename);
	for (int i=0; i<szLen; i++) {
		if (szFilename[i] == '(') {
			szIndx = ++i;
			while ((szFilename[i] != ')') && (i<szLen))
				i++;

			if (szFilename[i] == ')') {
				for (int n=szIndx; n<i; n++)
				{
					if ((szFilename[n] == 'J') || (szFilename[n] == 'U') || (szFilename[n] == 'j') || (szFilename[n] == 'u'))
					{
						setRegionMode(0);
						break;
					}
					else if ((szFilename[n] == 'E') || (szFilename[n] == 'e'))
					{
						setRegionMode(1);
						break;
					}
				}
			}
		}
	}
}

}
