
#include "Sound_Queue.h"

#include <assert.h>
#include <string.h>

/* Copyright (c) 2005 Shay Green. Permission is hereby granted, free
of charge, to any person obtaining a copy of this software and associated
documentation files (the "Software"), to deal in the Software without
restriction, including without limitation the rights to use, copy,
modify, merge, publish, distribute, sublicense, and/or sell copies of the
Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions: The above copyright notice and
this permission notice shall be included in all copies or substantial
portions of the Software. THE SOFTWARE IS PROVIDED "AS IS", WITHOUT
WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO
THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH
THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. */

Sound_Queue::Sound_Queue( long ms ) {
	// extra element allows queue full to look different than queue empty
	max_size = ms + 1;
	buf = new sample_t [max_size];
	write_pos = 0;
	read_pos = 0;
}

Sound_Queue::~Sound_Queue() {
	delete [] buf;
}

void Sound_Queue::clear() {
	write_pos = read_pos;
}

long Sound_Queue::buffer_avail() const {
	long max_avail = (read_pos + max_size - 1 - write_pos) % max_size;
	long avail = max_size - write_pos;
	return (avail < max_avail) ? avail : max_avail;
}

void Sound_Queue::write( long count ) {
	assert( count <= buffer_avail() );
	write_pos = (write_pos + count) % max_size;
}

void Sound_Queue::write( const sample_t* in, long count ) {
	while ( count ) {
		long n = buffer_avail();
		if ( n ) {
			if ( n > count )
				n = count;
			memcpy( buffer(), in, n * sizeof *in );
			in += n;
			count -= n;
			write( n );
		}
	}
}

long Sound_Queue::avail() const {
	return (write_pos + max_size - read_pos) % max_size;
}

long Sound_Queue::read( sample_t* out, long count )
{
	// samples available
	long total = avail();
	if ( count > total )
		count = total;
	total = count;
	
	// copy samples
	while ( count ) {
		long n = max_size - read_pos;
		if ( n > count )
			n = count;
		memcpy( out, buf + read_pos, n * sizeof *out );
		count -= n;
		read_pos = (read_pos + n) % max_size;
		out += n;
	}
	
	return total;
}

