enum logged_constants
{
	logged_opcode		= 0x0001,
	logged_oprand		= 0x0002,
	logged_immediate	= 0x0004,

	logged_Absolute1r	= 0x0008,
	logged_AbsoluteXr	= 0x0010,
	logged_AbsoluteYr	= 0x0020,
	logged_indirectXr	= 0x0040,
	logged_indirectYr	= 0x0080,

	logged_Absolute1w	= 0x0100,
	logged_AbsoluteXw	= 0x0200,
	logged_AbsoluteYw	= 0x0400,
	logged_indirectXw	= 0x0800,
	logged_indirectYw	= 0x1000,

	logged_Indirectjump	= 0x2000,
	logged_jumped		= 0x4000,
	logged_branched		= 0x8000
};

#define loggedOffset (pbankRead[cpu.pc>>11] - prgromPtr + (cpu.pc & 0x7FF))

/*
	p means its going to check for a page cross.
	temp must have the high address

	Check if temp is optimize, otherwise force it to pass the varible

	Note: will crash if one of these are in 2000-3FFF range
*/

namespace Reminesce
{

void inline AddcpuCycles(int cycles)
{
	cpuCycles += cycles;
}

void inline AddcpuCyclesp(int cycles)
{
	cpuCycles += (memloc.t16_.u.h == temp.t16_.u.h) ? cycles : cycles+1;
}

void inline adrImp()
{
	#ifdef loggedCpu
		if (cpu.pc >= 0x8000)
			loggedImage[ loggedOffset ]		|= logged_opcode;
		rwMode = 0;
	#endif
	cpu.pc++;
}
void inline adrImm()
{
	#ifdef loggedCpu
		if (cpu.pc >= 0x8000)
		{
			loggedImage[ loggedOffset]		|= logged_opcode;
			loggedImage[ loggedOffset+1]	|= logged_immediate;
		}
		rwMode = 1;
	#endif
	memloc.si_ = cpu.pc+1;
	cpu.pc += 2;
}

void inline adrZp()
{
	#ifdef loggedCpu
		if (cpu.pc >= 0x8000)
		{
			loggedImage[loggedOffset]		|= logged_opcode;
			loggedImage[loggedOffset+1]		|= logged_oprand;
		}
		rwMode = 2;
	#endif
	memloc.si_ = pRead(cpu.pc+1);
	cpu.pc += 2;
}

void inline adrZpx()
{
	#ifdef loggedCpu
	if (cpu.pc >= 0x8000)
		{
			loggedImage[loggedOffset]		|= logged_opcode;
			loggedImage[loggedOffset+1]		|= logged_oprand;
		}
		rwMode = 3;
	#endif
	memloc.si_ = (u8)(pRead(cpu.pc+1) + cpu.x);
	cpu.pc += 2;
}

void inline adrZpy()
{
	#ifdef loggedCpu
		if (cpu.pc >= 0x8000)
		{
			loggedImage[loggedOffset]		|= logged_opcode;
			loggedImage[loggedOffset+1]		|= logged_oprand;
		}
		rwMode = 4;
	#endif
	memloc.si_ = (u8)(pRead(cpu.pc+1) + cpu.y);
	cpu.pc += 2;
}

void inline adrAb()
{
	#ifdef loggedCpu
		if (cpu.pc >= 0x8000)
		{
			loggedImage[loggedOffset]		|= logged_opcode;
			loggedImage[loggedOffset+1]		|= logged_oprand;
			loggedImage[loggedOffset+2]		|= logged_oprand;
		}
		rwMode = 5;
	#endif
	memloc.si_  = pRead16(cpu.pc+1);
	cpu.pc  += 3;
}
void inline adrAbx()
{
	#ifdef loggedCpu
		if (cpu.pc >= 0x8000)
		{
			loggedImage[loggedOffset]		|= logged_opcode;
			loggedImage[loggedOffset+1]		|= logged_oprand;
			loggedImage[loggedOffset+2]		|= logged_oprand;
		}
		rwMode = 6;
	#endif
	memloc.si_  = pRead16(cpu.pc+1) + cpu.x;
	cpu.pc  += 3;
}

void inline adrAby()
{
	#ifdef loggedCpu
		if (cpu.pc >= 0x8000)
		{
			loggedImage[loggedOffset]		|= logged_opcode;
			loggedImage[loggedOffset+1]		|= logged_oprand;
			loggedImage[loggedOffset+2]		|= logged_oprand;
		}
		rwMode = 7;
	#endif
	memloc.si_  = pRead16(cpu.pc+1) + cpu.y;
	cpu.pc  += 3;
}

/* p means its going to check for a page cross.
temp must have the high address*/
void inline adrAbxp()
{
	#ifdef loggedCpu
		if (cpu.pc >= 0x8000)
		{
			loggedImage[loggedOffset]		|= logged_opcode;
			loggedImage[loggedOffset+1]		|= logged_oprand;
			loggedImage[loggedOffset+2]		|= logged_oprand;
		}
		rwMode = 8;
	#endif
	memloc.si_	 = pRead(cpu.pc+1);
	temp.si_	 = pRead(cpu.pc+2);
	memloc.si_	|= temp.si_<<8;
	memloc.si_	+= cpu.x;
	cpu.pc  += 3;
}

void inline adrAbyp()
{
	#ifdef loggedCpu
		if (cpu.pc >= 0x8000)
		{
			loggedImage[loggedOffset]		|= logged_opcode;
			loggedImage[loggedOffset+1]		|= logged_oprand;
			loggedImage[loggedOffset+2]		|= logged_oprand;
		}
		rwMode = 9;
	#endif
	memloc.si_	= pRead(cpu.pc+1);
	temp.si_		= pRead(cpu.pc+2);
	memloc.si_	|= temp.si_<<8;
	memloc.si_	+= cpu.y;
	cpu.pc  += 3;
}

void inline adrInx()
{
	#ifdef loggedCpu
		if (cpu.pc >= 0x8000)
		{
			loggedImage[loggedOffset]		|= logged_opcode;
			loggedImage[loggedOffset+1]		|= logged_oprand;
		}
		rwMode = 10;
	#endif
	temp.si_     = (u8)(pRead(cpu.pc+1) + cpu.x);
	memloc.si_  = pReadZp16(temp.si_);
	cpu.pc  += 2;
}

void inline adrIny()
{
	#ifdef loggedCpu
		if (cpu.pc >= 0x8000)
		{
			loggedImage[loggedOffset]		|= logged_opcode;
			loggedImage[loggedOffset+1]		|= logged_oprand;
		}
		rwMode = 11;
	#endif
	temp.si_     = pRead(cpu.pc+1);
	memloc.si_  = pReadZp16(temp.si_) + cpu.y;
	cpu.pc  += 2;
}

void inline adrInyp()
{
	#ifdef loggedCpu
		if (cpu.pc >= 0x8000)
		{
			loggedImage[loggedOffset]		|= logged_opcode;
			loggedImage[loggedOffset+1]		|= logged_oprand;
		}
		rwMode = 12;
	#endif
	temp.si_     = pRead(cpu.pc+1);
	memloc.si_  = pbankRead[0][(u8)temp.si_];
	temp.si_     = pbankRead[0][(u8)(temp.si_+1)];
	memloc.si_ |= temp.si_<<8;
	memloc.si_ += cpu.y;
	cpu.pc  += 2;
}

}
